//Temperaturregelung in Hühnerbrutkasten
//Proportionalregelung der Temperatur mit dem Messwert des I2C-Temperatursensors
//TH02. Als Heizung dient eine 12 V Halogenlampe (35 W), welche maximal mit 9 V betrieben wird.
//Die 9 V Versorgungsspannung für die Heizlampe wird über einen MOS-FET moduliert.
//Version für Visualisierung Daten mit Pythonprogramm "liveScopeTempFeuchtLeist_1.py"
//istTemp, istFeucht und heizLeist werden als mit Kommata getrennter String gesendet.
//Mit OLED-Display für o.g. Werte und Zeit
//SSD1306 0.96” OLED Display mit 128×64 Pixel:
//GND > Ground
//VCC > 5 V
//D0 > CLK von SPI = Arduino Pin 13
//D1 > MOSI = Arduino Pin 11
//RES = Arduino Pin 9
//DC = Arduino Pin 8
//CS = Arduino Pin 10
//Warteschleife in setup() entfernt, in der Arduino auf Serial Event wartet. Dadurch startet
//das Programm auch ohne Input über die serielle Schnittstelle des angeschlossenen PC.
//Die im Display dargestellte Zeit wird nicht mehr über millis() sondern über den inkrementierten
//Zähler min_passed erfasst. Bei millis() fand schon zu früh ein Überlauf statt.
//Bug beseitig beim Sekundenzähler ((i*COM_PER) erzeugte Overflow). Blinkendes :-Zeichen als
//Lebenszeichen der Software eingeführt.
//Die Bibliothek für den TH02-Sensor wurde noch nicht installiert, da sich die Arduino-IDE wegen
//Sonderzeichen im Namen mockierte. Daher hier im selben Verzeichnis als TH02_dev.cpp bzw.
//TH02_dev.h präsent.
//S. Mack, 22.03.20


#include "TH02_dev.h"
#include <Wire.h>
#include <SPI.h>
#include <HT_SSD1306.h>
#define PIN_RESET 9  // Connect RST to pin 9
#define PIN_DC    8  // Connect DC to pin 8
#define PIN_CS    10 // Connect CS to pin 10

float istTemp=0.0, istFeucht=0.0, heizLeist=0.0;
const float SOLLTEMP = 37.0, SOLLFEUCHT = 70.0;
const int GAIN = 1; //je kleiner der Wert desto empfindlicher die T-Regelung
const int COM_PER = 20; // Periode für Kommunikation in ms
int i=0; // wird mit Periode COM_PER inkrementiert
unsigned int min_passed = 0; // Reicht 45 Tage, 12 h und 15 min
boolean toggle = true; // fuer blinkendes :-zeichen im Display

SSD1306 oled(PIN_RESET, PIN_DC, PIN_CS);

void setup()
{  
  pinMode(3, OUTPUT);  
  delay(150);
  TH02.begin();
  oled.begin();
  oled.clear(ALL);
  delay(100);
    int i=0;
  // Initialwerte, damit als erster Wert nicht Null ausgegeben wird
  istTemp = TH02.ReadTemperature();
  istFeucht = TH02.ReadHumidity();
  
  Serial.begin(115200); 
}

// Kommunikation zu Pythonskript ist alle COM_PER = 20 ms möglich. 
// Variable i wird damit inkrementiert. Messwerterfassung und Regelung findet 
// nur jede Sekunde(falls (i*COM_PER)%1000 == 0) statt.
void loop() {
//Eine Sekunde ist vorbei
if (i%(1000/COM_PER)==0) {
  //Serial.print("i=");
  //Serial.println(i);
  //Serial.println("Wieder eine Sekunde vorbei...");
  istTemp = TH02.ReadTemperature();
  istFeucht = TH02.ReadHumidity();
  heizLeist = constrain(((SOLLTEMP - istTemp)/GAIN),0,1);
  analogWrite (3, int(heizLeist*255));
  update_disp(istTemp, istFeucht, heizLeist, min_passed);  
  oled.display();
  }

// Falls vom PC ein Messwert angefordert wurde (= Zeichen an serieller Schnittstelle liegt an)
if (Serial.available()>0)  {
  // Schnittstelle auslesen
  int inByte = Serial.read();
  Serial.print(int(istTemp*10),DEC);
  Serial.print(',');
  Serial.print(int(istFeucht),DEC);
  Serial.print(',');
  Serial.println(int(heizLeist*100),DEC);
  }
delay(COM_PER);
i++;
//Serial.print('.');
// Eine Minute ist vorbei
if(i%(60000/COM_PER)==0) {
  min_passed++;
  //Serial.println('.');
  //Serial.print("Eine Minute vorbei: ");
  //Serial.println(min_passed);
  i=0;
  }
}

void update_disp(float istTemp, float istFeucht, float heizLeist, int min_passed)
{  
  oled.clear(PAGE);
  oled.setFontType(1); // Font 1 ist 16 Pixel hoch 8 breit 
  oled.setCursor(0,0);
  oled.print("Temp   : ");
  oled.print(istTemp);
  oled.setCursor(0, 16);
  oled.print("Feuchte: ");
  oled.print(istFeucht);
  oled.setCursor(0, 32);
  oled.print("Heizung: ");
  oled.print(heizLeist*100);
  oled.setCursor(0, 48);
  oled.print("Zeit"); // :-Zeichen blinkt als Lebenszeichen der Software
  if (toggle) {oled.print(": ");} else {oled.print("  ");}
  toggle = !toggle; 
  oled.print(min_passed / (24*60)); // Tage
  oled.print(":");
  oled.print((min_passed / 60) % 24); // Stunden
  oled.print(":");
  oled.print(min_passed % 60); // Minuten
  oled.display();
}
