#!/usr/bin/python3
# Live Plot der Messdaten eines Arduinos mit matplotlib
# Aufbauend auf Bspl. 'Oscilloscope' aus Matplotlib User Guide  J. Hunter (S. 508)
# Besser als Beispiel aus Raspberry Pi Cookbook for Python Programmers,
# welches für liveGraphArduAdcScope_1.py verwendet wird.
# blit=False in animation() noetig, damit x Achse aktualisiert wird. Beim
# schnellerem Sampeln gibt es hier vielleicht Probleme.
# Noch nicht gelöst: Schießen des Plotfensters ohne Exception im Kommandofenster.
# Funktioniert zusammen mit Arduino-Sketch 
# +++++++++++++++++ TempRegelBrutDisp_SerComOptional_1.ino ++++++++++++++++
# S. Mack, 22.3.20

from matplotlib.lines import Line2D
import matplotlib.pyplot as plt
import matplotlib.animation as animation
from time import sleep
import logging
import serial
from sys import exit

#logging.basicConfig(level=10)

SAMPTIME=10 # Samplingzeit in s
TIMESPAN=600 # Dauer Aufzeichnungsintervalls in s, vor neuer Bildaufbau
YPAD=2 # Y-Randbereich Plot über und unter min/max Daten

arduVal=0
#maxt: Im Plot dargest Zeitspanne in s, maxy: Initiale Länge
# y-Achse, wird spaeter angepasst. dt: Samplingintervall
class Scope(object):

    def __init__(self, ax, maxt=10, maxy=100, dt=1): 
        self.ax = ax
        self.dt = dt
        self.maxt = maxt
        self.maxy = maxy
        self.tdata = [0]
        dummy = getArduVal()
        self.y0data = [dummy[0]] # damit y(t=0) nicht 0
        self.y1data = [dummy[1]]
        self.y2data = [dummy[2]]
        self.line0 = Line2D(self.tdata, self.y0data, color = 'C0')
        self.line0.set_label('Temperatur (°C)')
        self.ax.add_line(self.line0)
        self.line1 = Line2D(self.tdata, self.y1data, color = 'C1')
        self.line1.set_label('Feuchtigkeit (%)')
        self.ax.add_line(self.line1)
        self.line2 = Line2D(self.tdata, self.y2data, color = 'C2')
        self.line2.set_label('Heizleistung (%)')
        self.ax.add_line(self.line2)
        self.ax.set_ylim(-YPAD, self.maxy+YPAD)
        self.ax.set_xlim(0, self.maxt)
        self.ax.set_title('Messwerte Brutkasten')
        self.ax.set_xlabel('Zeit (s)')
        self.ax.set_ylabel('Werte')       
        self.ax.grid(linestyle='--') # gestrichelte Gitterlinien
        self.ax.legend() # Legende darstellen
        
    def update(self, y):
        logging.info('update()')
        lastt = self.tdata[-1]
        # wenn mehr als maxt Sekunden vergangen
        if lastt > self.tdata[0] + self.maxt:
            logging.info('time span over...')
            # jeweils letzes altes Element wird erstes neues Element in Liste
            self.tdata = [self.tdata[-1]]
            self.y0data = [self.y0data[-1]]
            self.y1data = [self.y1data[-1]]
            self.y2data = [self.y2data[-1]]
            #x-Achse neu Beschriften für naechste Zeitspanne
            self.ax.set_xlim(self.tdata[0], self.tdata[0] + self.maxt)
            self.ax.figure.canvas.draw()
        
        t = self.tdata[-1] + self.dt # aktuelle Zeit
        self.tdata.append(t) # Liste Zeiten um neue Messzeit erweitern
        self.y0data.append(y[0]) # Liste Werte um neuen Messwert erweitern
        self.y1data.append(y[1]) 
        self.y2data.append(y[2]) 
        logging.info('data: %s %s %s %s', self.tdata,self.y0data,self.y1data,self.y2data)
        self.line0.set_data(self.tdata, self.y0data)
        self.line1.set_data(self.tdata, self.y1data)
        self.line2.set_data(self.tdata, self.y2data)
        return self.line0, self.line1, self.line2

def getArduVal(): # Neue Werte serieller Schnittstelle lesen
    port.flushInput() # Seriellen Puffer leeren
    port.write(b'\n') # ADC-Messung Arduino anfordern
    sleep(0.2)
    result = port.readline() # Von Schnittstelle bis \n lesen (Byte Array)
    logging.info('getAdcVal(): %s', str(result))
    valString = result.decode()[:-2] # In String konvertieren
    valList = valString.split(',') # An Kommata auftrennen und in Liste eintragen
    result = list(map(float, valList)) # Aus Stringliste Integerliste machen
    result[0] = result[0]/10 # Einheit Grad Celsisus statt 10*
    print('Temperatur: {}, Feuchte: {}, Heizleistung: {}'.format(result[0],result[1],result[2]))
    return(result) # gibt Integerliste mit drei Werten zurück

def data_gen(): # Muss Iterator sein, daher die While-Schleife
    while True:
        logging.info('data_gen() inside')
        arduVal = getArduVal()
        yield arduVal    
      
try:
    port = serial.Serial(port='/dev/ttyUSB1', baudrate=115200, timeout=0.2)
    # Erster Lesevorgang zum Reset des Arduino und falls etwas an der Schnittstelle anliegt
    # Falls nichts anliegt greift der Timeout von 0,2 s
    print('Arduino sagt: ', port.readline()) # Testen ob Arduino bereit, String auslesen
    # Plot und Achse erstellen
    fig, ax = plt.subplots()
    scope = Scope(ax=ax,maxt=TIMESPAN,dt=SAMPTIME)
    # Animation instanzieren,intervall in ms, blit=False zum Nachskalieren x-Achse nötig 
    ani = animation.FuncAnimation(fig, scope.update, data_gen, interval=(SAMPTIME*1000), blit=False)
    print('Animation instanziert...')
    plt.show() # Plot starten (Programm bleibt ab jetzt in dieser Schleife bis Plot geschlossen)     
except KeyboardInterrupt:
    print()
    print('Strg + C erkannt...')      
finally:
    print('...COM-Verbindung wird beendet.')
    port.close() # COM-Verbindung beenden
    sleep(1)
    exit(0)

